#include "MyWindow.h"

/**
 * Constructor: create an image
 */
MyWindow::MyWindow() 
{
	m_width		= 1200;		// width and height of the image
    m_height	= 900;
    m_iters		= 1000000;
    
    m_minX		= -2.1;
    m_minY		= m_minX * m_height / m_width;
    m_maxX		= 2.1;
    m_maxY		= m_maxX * m_height / m_width;

	m_a			= 1.4;
    m_b			= -1.8;
    m_c			= 1.1;
    m_d			= -0.3;

	// reserve some space for our colours
	m_imageColor.reserve( m_width * m_height );
	for( int i = 0; i < m_width * m_height; i++ )
	{ 
		m_imageColor.push_back( Color::black() );
	}

	// create an image
	m_image.Create( m_width, m_height );
	
	// fill image
	for ( long i = 0; i < m_image.Width(); i++ )
	{
		for ( long j = 0; j < m_image.Height(); j++ )
		{
            Color &c = m_imageColor[ i + j * m_width ];
			m_image.SetPixel3f( i, j, (float)c.r, (float)c.g, (float)c.b );
		}
	}

	// resize window to match image
	SetSize( m_image.Width(), m_image.Height() );
}

//-----------------------------------------------------------------------------

/**
 * Display function
 */
void MyWindow::OnDisplay() 
{
	glClear( GL_COLOR_BUFFER_BIT );

	// draw the image into the window, 1:1 size, bottom-left corner
	glRasterPos2f( -1, -1 );
	m_image.glDrawPixels();

	SwapBuffers();
}

//-----------------------------------------------------------------------------

/**
 * works out the fractal colours and draws it
 */
void MyWindow::Draw() 
{
    double x = 0.0;
	double y = 0.0;
	double xn = 0.0;
	double yn = 0.0;

	Color addColor = Color(0.1, 0.1, 0.1);

	cout << "Generating colors" << endl;

	for( int i = 0; i < m_iters; i++ )
	{
		xn	= sin( m_a * y ) + m_c * cos( m_a * x );
		yn	= sin( m_b * x ) + m_d * cos( m_b * y );
		x	= xn;
		y	= yn;        
	    
		int xi = static_cast<int>( ( x - m_minX ) * m_width / ( m_maxX - m_minX ) );
		int yi = static_cast<int>( ( y - m_minY ) * m_height / ( m_maxY - m_minY ) );
		if ( xi >= 0 && xi < m_width &&
			 yi >= 0 && yi < m_height ) 
		{
			m_imageColor[ xi + yi * m_width ] += addColor;
		}
		cout << "\r" << i << "/" << m_iters;
	}
	cout << "Finished generating colors" << endl;

	for ( int y = 0; y < m_height; y++ ) 
	{
        for ( int x = 0; x < m_width; x++ ) 
		{
            Color &c = m_imageColor[ x + y * m_width ];
			m_image.SetPixel3f( x, y, (float)c.r, (float)c.g, (float)c.b );
		}
	}

	cout << "Done" << endl;
}

//-----------------------------------------------------------------------------

/**
 * Redraw continuously on idle
 */
void MyWindow::OnIdle() 
{
	Redraw();
}

//-----------------------------------------------------------------------------

/*
 * some keyboard controls
 */
void MyWindow::OnKeyboard(int key, bool down) 
{
	if ( !down ) return;
	//key pressed
	switch( tolower( key ) ) 
	{
		case 'q':
			Close();
			break;
		case 's':
			m_image.Save( "attractor.jpg" );
			break;
		default:
			break;
	}
}	